@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2001 - 2007 Rob Caelers <robc@krandor.nl>
+// Copyright (C) 2001 - 2010 Rob Caelers <robc@krandor.nl>
 // Copyright (C) 2007 Ray Satiro <raysatiro@yahoo.com>
 // All rights reserved.
 //
@@ -138,6 +138,8 @@ ActivityMonitor::force_idle()
   if (activity_state != ACTIVITY_SUSPENDED)
     {
       activity_state = ACTIVITY_IDLE;
+      last_action_time.tv_sec = 0;
+      last_action_time.tv_usec = 0;
     }
   lock.unlock();
   TRACE_RETURN(activity_state);
@@ -315,10 +317,8 @@ ActivityMonitor::mouse_notify(int x, int y, int wheel_delta)
 
 //! Mouse button activity is reported by the input monitor.
 void
-ActivityMonitor::button_notify(int button_mask, bool is_press)
+ActivityMonitor::button_notify(bool is_press)
 {
-  (void)button_mask;
-
   lock.lock();
 
   button_is_pressed = is_press;
@@ -334,10 +334,9 @@ ActivityMonitor::button_notify(int button_mask, bool is_press)
 
 //! Keyboard activity is reported by the input monitor.
 void
-ActivityMonitor::keyboard_notify(int key_code, int modifier)
+ActivityMonitor::keyboard_notify(bool repeat)
 {
-  (void)key_code;
-  (void)modifier;
+  (void)repeat;
 
   lock.lock();
   action_notify();
