@@ -1,6 +1,6 @@
 // Core.cc --- The main controller
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -89,6 +89,7 @@ Core::Core() :
   application(NULL),
   statistics(NULL),
   operation_mode(OPERATION_MODE_NORMAL),
+  usage_mode(USAGE_MODE_NORMAL),
   core_event_listener(NULL),
   powersave(false),
   powersave_resume_time(0),
@@ -406,6 +407,26 @@ Core::config_changed_notify(const string &key)
     {
       load_monitor_config();
     }
+
+  if (key == CoreConfig::CFG_KEY_OPERATION_MODE)
+    {
+      int mode;
+      if (! get_configurator()->get_value(CoreConfig::CFG_KEY_OPERATION_MODE, mode))
+        {
+          mode = OPERATION_MODE_NORMAL;
+        }
+      set_operation_mode(OperationMode(mode), false);
+    }
+
+  if (key == CoreConfig::CFG_KEY_USAGE_MODE)
+    {
+      int mode;
+      if (! get_configurator()->get_value(CoreConfig::CFG_KEY_USAGE_MODE, mode))
+        {
+          mode = USAGE_MODE_NORMAL;
+        }
+      set_usage_mode(UsageMode(mode), false);
+    }
 }
 
 
@@ -539,12 +560,11 @@ Core::set_operation_mode(OperationMode mode, bool persistent)
           monitor->suspend();
           stop_all_breaks();
           
-          for( int i = 0; i < BREAK_ID_SIZEOF; ++i )
+          for (int i = 0; i < BREAK_ID_SIZEOF; ++i)
             {
-              if( breaks[ i ].is_enabled() )
+              if (breaks[i].is_enabled())
                 {
-                  breaks[ i ].get_timer()->set_insensitive_mode( MODE_IDLE_ALWAYS );
-                  //breaks[ i ].get_timer()->freeze_timer( true );
+                  breaks[i].get_timer()->set_insensitive_mode(INSENSITIVE_MODE_IDLE_ALWAYS);
                 }
             }
         }
@@ -555,7 +575,7 @@ Core::set_operation_mode(OperationMode mode, bool persistent)
           monitor->resume();
         }
       
-      if( operation_mode == OPERATION_MODE_QUIET )
+      if (operation_mode == OPERATION_MODE_QUIET)
         {
           stop_all_breaks();
         }
@@ -576,6 +596,31 @@ Core::set_operation_mode(OperationMode mode, bool persistent)
 }
 
 
+UsageMode
+Core::get_usage_mode()
+{
+  return usage_mode;
+}
+
+void
+Core::set_usage_mode(UsageMode mode, bool persistent)
+{
+  if (usage_mode != mode)
+    {
+      usage_mode = mode;
+  
+      for (int i = 0; i < BREAK_ID_SIZEOF; i++)
+        {
+          breaks[i].set_usage_mode(mode);
+        }
+
+      if (persistent)
+        {
+          get_configurator()->set_value(CoreConfig::CFG_KEY_USAGE_MODE, mode);
+        }
+    }      
+}
+
 //! Sets the listener for core events.
 void
 Core::set_core_events_listener(ICoreEventListener *l)
@@ -586,19 +631,19 @@ Core::set_core_events_listener(ICoreEventListener *l)
 
 //! Forces the start of the specified break.
 void
-Core::force_break(BreakId id, bool initiated_by_user)
+Core::force_break(BreakId id, BreakHint break_hint)
 {
-  do_force_break(id, initiated_by_user);
+  do_force_break(id, break_hint);
 
 #ifdef HAVE_DISTRIBUTION
-  send_break_control_message_bool_param(id, BCM_START_BREAK, initiated_by_user);
+  send_break_control_message_bool_param(id, BCM_START_BREAK, break_hint);
 #endif
 }
 
 
 //! Forces the start of the specified break.
 void
-Core::do_force_break(BreakId id, bool initiated_by_user)
+Core::do_force_break(BreakId id, BreakHint break_hint)
 {
   TRACE_ENTER_MSG("Core::do_force_break", id);
   BreakControl *microbreak_control = breaks[BREAK_ID_MICRO_BREAK].get_break_control();
@@ -612,7 +657,7 @@ Core::do_force_break(BreakId id, bool initiated_by_user)
       TRACE_MSG("Resuming Micro break");
     }
 
-  breaker->force_start_break(initiated_by_user);
+  breaker->force_start_break(break_hint);
   TRACE_EXIT();
 }
 
@@ -716,6 +761,8 @@ Core::force_idle()
         {
           am->force_idle();
         }
+
+      breaks[i].get_timer()->force_idle();
     }
 }
 
@@ -815,18 +862,21 @@ Core::heartbeat()
   // Set current time.
   current_time = time(NULL);
 
+  // Performs timewarp checking.
+  bool warped = process_timewarp();
+
   // Process configuration
   configurator->heartbeat();
 
   // Perform distribution processing.
   process_distribution();
 
-  // Perform state computation.
-  process_state();
-
-  // Performs timewarp checking.
-  process_timewarp();
-
+  if (!warped)
+    {
+      // Perform state computation.
+      process_state();
+    }
+  
   // Perform timer processing.
   process_timers();
 
@@ -1002,7 +1052,7 @@ void
 Core::get_timer_overdue(BreakId id, int *value)
 {
   Timer *timer = get_timer(id);
-  *value = timer->get_total_overdue_time();
+  *value = (int) timer->get_total_overdue_time();
 }
 
 //! Processes all timers.
@@ -1042,6 +1092,7 @@ Core::process_timers()
         }
     }
 
+  
   // Process all timer events.
   for (int i = BREAK_ID_SIZEOF - 1; i >= 0;  i--)
     {
@@ -1068,9 +1119,11 @@ Core::process_timers()
 #ifdef PLATFORM_OS_WIN32
 
 //! Process a possible timewarp on Win32
-void
+bool
 Core::process_timewarp()
 {
+  bool ret = false;
+  
   TRACE_ENTER("Core::process_timewarp");
   if (last_process_time != 0)
     {
@@ -1081,7 +1134,6 @@ Core::process_timewarp()
             {
               TRACE_MSG("Time warp of " << gap << " seconds. Correcting");
 
-
               force_idle();
 
               monitor->shift_time((int)gap);
@@ -1091,12 +1143,12 @@ Core::process_timewarp()
                 }
 
               monitor_state = ACTIVITY_IDLE;
+              ret = true;
             }
           else
             {
               TRACE_MSG("Time warp of " << gap << " seconds because of powersave");
 
-
               // In case the windows message was lost. some people reported that
               // workrave never restarted the timers...
               set_operation_mode(powersave_operation_mode);
@@ -1106,47 +1158,33 @@ Core::process_timewarp()
         {
           TRACE_MSG("End of time warp after powersave");
 
-
           powersave = false;
           powersave_resume_time = 0;
-
         }
     }
   TRACE_EXIT();
+  return ret;
 }
 
 #else
 
 //! Process a possible timewarp On Non-Windows
-void
+bool
 Core::process_timewarp()
 {
+  bool ret = false;
+  
   TRACE_ENTER("Core::process_timewarp");
   if (last_process_time != 0)
     {
       int gap = current_time - 1 - last_process_time;
-      bool do_powersafe = false;
-      bool do_timeshift = false;
 
-      if (gap < 0 || (gap > 5 && gap < 600))
-        {
-          // Negative gap are always intrepreted as timechanges.
-          // Positive gaps between 5..600 are also considered timechanges
-          do_timeshift = true;
-        }
-      else if (gap >= 600)
-        {
-          // Gaps > 600 are considered powersaves.
-          do_powersafe = true;
-        }
-      
-      if (do_powersafe)
+      if (gap >= 30)
         {
           TRACE_MSG("Time warp of " << gap << " seconds. Powersafe");
           
           force_idle();
 
-          /* Assume it is a powersave... */
           int save_current_time = current_time;
 
           current_time = last_process_time + 1;
@@ -1155,21 +1193,12 @@ Core::process_timewarp()
           process_timers();
 
           current_time = save_current_time;
-        }
-
-      if (do_timeshift)
-        {
-          TRACE_MSG("Time warp of " << gap << " seconds. Timeshift");
-
-          monitor->shift_time(gap);
-          for (int i = 0; i < BREAK_ID_SIZEOF; i++)
-            {
-              breaks[i].get_timer()->shift_time(gap);
-            }
+          ret = true;
         }
     }
   
   TRACE_EXIT();
+  return ret;
 }
 
 #endif
@@ -1203,6 +1232,10 @@ Core::timer_action(BreakId id, TimerInfo info)
         }
       break;
 
+    case TIMER_EVENT_NATURAL_RESET:
+      statistics->increment_break_counter(id, Statistics::STATS_BREAKVALUE_NATURAL_TAKEN);
+      // FALLTHROUGH
+      
     case TIMER_EVENT_RESET:
       if (breaker->get_break_state() == BreakControl::BREAK_ACTIVE)
         {
@@ -1210,14 +1243,6 @@ Core::timer_action(BreakId id, TimerInfo info)
         }
       break;
 
-    case TIMER_EVENT_NATURAL_RESET:
-      if (breaker->get_break_state() == BreakControl::BREAK_ACTIVE)
-        {
-          breaker->stop_break();
-        }
-      statistics->increment_break_counter(id, Statistics::STATS_BREAKVALUE_NATURAL_TAKEN);
-      break;
-
     default:
       break;
     }
@@ -1244,6 +1269,11 @@ Core::start_break(BreakId break_id, BreakId resume_this_break)
   BreakControl *restbreak_control;
   restbreak_control = breaks[BREAK_ID_REST_BREAK].get_break_control();
 
+  if (break_id == BREAK_ID_REST_BREAK && resume_this_break == BREAK_ID_NONE)
+    {
+      breaks[BREAK_ID_REST_BREAK].override(BREAK_ID_REST_BREAK);
+    }
+  
   if (break_id == BREAK_ID_MICRO_BREAK && breaks[BREAK_ID_REST_BREAK].is_enabled())
     {
       Timer *rb_timer = breaks[BREAK_ID_REST_BREAK].get_timer();
@@ -1264,6 +1294,8 @@ Core::start_break(BreakId break_id, BreakId resume_this_break)
 
           if (now + duration + 30 >= rb_timer->get_next_limit_time())
             {
+              breaks[BREAK_ID_REST_BREAK].override(BREAK_ID_MICRO_BREAK);
+                
               start_break(BREAK_ID_REST_BREAK, BREAK_ID_MICRO_BREAK);
 
               // Snooze timer before the limit was reached. Just to make sure
@@ -1381,12 +1413,21 @@ Core::save_state() const
 void
 Core::load_misc()
 {
+  configurator->add_listener(CoreConfig::CFG_KEY_OPERATION_MODE, this);
+  configurator->add_listener(CoreConfig::CFG_KEY_USAGE_MODE, this);
+  
   int mode;
   if (! get_configurator()->get_value(CoreConfig::CFG_KEY_OPERATION_MODE, mode))
     {
       mode = OPERATION_MODE_NORMAL;
     }
   set_operation_mode(OperationMode(mode), false);
+
+  if (! get_configurator()->get_value(CoreConfig::CFG_KEY_USAGE_MODE, mode))
+    {
+      mode = USAGE_MODE_NORMAL;
+    }
+  set_usage_mode(UsageMode(mode), false);
 }
 
 
@@ -1459,7 +1500,7 @@ Core::post_event(CoreEvent event)
 void
 Core::freeze()
 {
-  TRACE_ENTER_MSG("BreakControl::freeze", insist_policy);
+  TRACE_ENTER_MSG("Core::freeze", insist_policy);
   ICore::InsistPolicy policy = insist_policy;
 
   switch (policy)
@@ -1494,7 +1535,7 @@ Core::freeze()
 void
 Core::defrost()
 {
-  TRACE_ENTER_MSG("BreakControl::defrost", active_insist_policy);
+  TRACE_ENTER_MSG("Core::defrost", active_insist_policy);
 
   switch (active_insist_policy)
     {
@@ -1922,15 +1963,21 @@ Core::set_break_control(PacketBuffer &buffer)
           break;
 
         case BCM_START_BREAK:
-          if (data_size >= 5)
+          if (data_size >= 6)
+            {
+              // Only for post 1.9.1 workrave...
+              int break_hint = (int) buffer.unpack_ushort();
+              do_force_break(break_id, (BreakHint) break_hint);
+            }
+          else if (data_size >= 5)
             {
               // Only for post 1.6.2 workrave...
               bool initiated_by_user = (bool) buffer.unpack_byte();
-              do_force_break(break_id, initiated_by_user);
+              do_force_break(break_id, initiated_by_user ? BREAK_HINT_USER_INITIATED : BREAK_HINT_NONE);
             }
           else
             {
-              do_force_break(break_id, true);
+              do_force_break(break_id, BREAK_HINT_USER_INITIATED);
             }
           break;
         }
