@@ -1,6 +1,6 @@
 // Statistics.cc
 //
-// Copyright (C) 2002 - 2008 Rob Caelers & Raymond Penners
+// Copyright (C) 2002 - 2008, 2010 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -46,6 +46,7 @@
 const char *WORKRAVESTATS="WorkRaveStats";
 const int STATSVERSION = 4;
 
+#define MAX_JUMP (10000)
 
 //! Constructor
 Statistics::Statistics() :
@@ -547,7 +548,7 @@ Statistics::set_counter(StatsValueType t, int value)
 }
 
 
-int
+int64_t
 Statistics::get_counter(StatsValueType t)
 {
   return current_day->misc_stats[t];
@@ -579,7 +580,7 @@ Statistics::dump()
   ss << "stats ";
   for(int j = 0; j < STATS_VALUE_SIZEOF; j++)
     {
-      int value = current_day->misc_stats[j];
+      int64_t value = current_day->misc_stats[j];
 
       ss  << value << " ";
     }
@@ -959,24 +960,32 @@ Statistics::mouse_notify(int x, int y, int wheel_delta)
 
   lock.lock();
 
-  if (current_day != NULL)
+  if (current_day != NULL && x >=0 && y >= 0)
     {
       int delta_x = sensitivity;
       int delta_y = sensitivity;
 
       if (prev_x != -1 && prev_y != -1)
         {
-          delta_x = x - prev_x;
-          delta_y = y - prev_y;
+          delta_x = abs(x - prev_x);
+          delta_y = abs(y - prev_y);
         }
       
       prev_x = x;
       prev_y = y;
 
-      if (abs(delta_x) >= sensitivity || abs(delta_y) >= sensitivity || wheel_delta != 0)
+      // Sanity checks, ignore unreasonable large jumps...
+      if ( delta_x < MAX_JUMP && delta_y < MAX_JUMP &&
+          (delta_x >= sensitivity || delta_y >= sensitivity || wheel_delta != 0 ))
         {
-          current_day->misc_stats[STATS_VALUE_TOTAL_MOUSE_MOVEMENT]  +=
-            int(sqrt((double)(delta_x * delta_x + delta_y * delta_y)));
+          int64_t movement = current_day->misc_stats[STATS_VALUE_TOTAL_MOUSE_MOVEMENT];
+          int distance = int(sqrt((double)(delta_x * delta_x + delta_y * delta_y)));
+
+          movement += distance;
+          if (movement > 0)
+            {
+              current_day->misc_stats[STATS_VALUE_TOTAL_MOUSE_MOVEMENT] = movement;
+            }
 
           GTimeVal now, tv;
 
@@ -1001,10 +1010,8 @@ Statistics::mouse_notify(int x, int y, int wheel_delta)
 
 //! Mouse button activity is reported by the input monitor.
 void
-Statistics::button_notify(int button_mask, bool is_press)
+Statistics::button_notify(bool is_press)
 {
-  (void)button_mask;
-
   lock.lock();
   if (current_day != NULL)
     {
@@ -1014,8 +1021,14 @@ Statistics::button_notify(int button_mask, bool is_press)
           int delta_x = click_x - prev_x;
           int delta_y = click_y - prev_y;
 
-          current_day->misc_stats[STATS_VALUE_TOTAL_CLICK_MOVEMENT] +=
-            int(sqrt((double)(delta_x * delta_x + delta_y * delta_y)));
+          int64_t movement = current_day->misc_stats[STATS_VALUE_TOTAL_CLICK_MOVEMENT];
+          int64_t distance = int(sqrt((double)(delta_x * delta_x + delta_y * delta_y)));
+
+          movement += distance;
+          if (movement > 0)
+            {
+              current_day->misc_stats[STATS_VALUE_TOTAL_CLICK_MOVEMENT] = movement;
+            }
         }
 
       click_x = prev_x;
@@ -1032,10 +1045,10 @@ Statistics::button_notify(int button_mask, bool is_press)
 
 //! Keyboard activity is reported by the input monitor.
 void
-Statistics::keyboard_notify(int key_code, int modifier)
+Statistics::keyboard_notify(bool repeat)
 {
-  (void)key_code;
-  (void)modifier;
+  if (repeat)
+    return;
 
   lock.lock();
   if (current_day != NULL)
