@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2002-2008 Raymond Penners <raymond@dotsphinx.com>
  * Copyright (C) 2007 Ray Satiro <raysatiro@yahoo.com>
+ * Copyright (C) 2009-2010 Rob Caelers <robc@krandor.nl>
  * All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
@@ -20,6 +21,8 @@
  *
  */
 
+#define _CRT_SECURE_NO_WARNINGS
+
 #include <windows.h>
 
 #include <stdio.h>
@@ -50,12 +53,12 @@ HHOOK DLLSHARE(mouse_hook) = NULL;
 HHOOK DLLSHARE(mouse_ll_hook) = NULL;
 HHOOK DLLSHARE(keyboard_hook) = NULL;
 HHOOK DLLSHARE(keyboard_ll_hook) = NULL;
+HHOOK DLLSHARE(msg_hook) = NULL;
 BOOL DLLSHARE(block_input) = FALSE;
 BOOL DLLSHARE(initialized) = FALSE;
 char DLLSHARE( critical_file_list[ HARPOON_MAX_UNBLOCKED_APPS ][ 511 ] ) = { 0, };
 HWND DLLSHARE( debug_hwnd ) = NULL;
 int DLLSHARE( debug ) = FALSE;
-//int DLLSHARE( harpoon_supports_mouse_hook_struct_ex ) = FALSE;
 
 #if !defined(__GNUC__)
 #pragma data_seg()
@@ -64,6 +67,7 @@ int DLLSHARE( debug ) = FALSE;
 #define IDM_MENU_SAVE 0
 #define IDM_MENU_MONITOR 1
 #define IDM_MENU_CLEAR 2
+#define IDM_MENU_UNHOOK 3
 
 // Each instance of harpoon should have the
 // filename of the process it is attached to.
@@ -255,7 +259,7 @@ harpoon_window_proc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
             {
             case HARPOON_KEY_PRESS:
             case HARPOON_KEY_RELEASE:
-              evt.keyboard.flags = lParam;
+              evt.keyboard.flags = (int) lParam;
               break;
 
             case HARPOON_BUTTON_PRESS:
@@ -275,11 +279,11 @@ harpoon_window_proc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
               if (evt.type == HARPOON_MOUSE_WHEEL)
                 {
                   evt.mouse.button = -1;
-                  evt.mouse.wheel = wParam;
+                  evt.mouse.wheel = (int) wParam;
                 }
               else
                 {
-                  evt.mouse.button = wParam;
+                  evt.mouse.button = (int) wParam;
                   evt.mouse.wheel = 0;
                 }
               break;
@@ -304,7 +308,7 @@ harpoon_window_proc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
  * Mouse hook
  **********************************************************************/
 
-LRESULT CALLBACK
+static LRESULT CALLBACK
 harpoon_mouse_hook (int code, WPARAM wpar, LPARAM lpar)
 {
   BOOL forcecallnext = FALSE;
@@ -475,7 +479,7 @@ harpoon_mouse_hook (int code, WPARAM wpar, LPARAM lpar)
                                       forcecallnext);
 }
 
-LRESULT CALLBACK
+static LRESULT CALLBACK
 harpoon_mouse_ll_hook (int code, WPARAM wpar, LPARAM lpar)
 {
   if (code == HC_ACTION)
@@ -555,13 +559,15 @@ harpoon_mouse_ll_hook (int code, WPARAM wpar, LPARAM lpar)
 }
 
 
-LRESULT CALLBACK
+static LRESULT CALLBACK
 harpoon_mouse_block_hook (int code, WPARAM wpar, LPARAM lpar)
 {
   BOOL forcecallnext = FALSE;
 
   if (code == HC_ACTION)
     {
+      if_debug_send_message( "WH_MOUSE" );
+
       switch (wpar)
         {
         case WM_LBUTTONUP:
@@ -689,6 +695,50 @@ harpoon_keyboard_block_hook (int code, WPARAM wpar, LPARAM lpar)
 }
 
 
+static LRESULT CALLBACK 
+harpoon_msg_block_hook(int code, WPARAM wpar, LPARAM lpar)
+{
+  BOOL forcecallnext = TRUE;
+
+  if (code >= 0)
+    {
+       //if (is_app_blocked() && block_input)
+       //{
+       //   ((MSG*)lpar)->message = WM_NULL;
+       //}
+
+       if (((MSG*)lpar)->message == WM_KEYDOWN)
+        {
+          if (is_app_blocked() && block_input)
+            {
+              ((MSG*)lpar)->message = WM_NULL;
+            }
+        }
+
+      if ( ((MSG*)lpar)->message >= WM_MOUSEFIRST &&
+           ((MSG*)lpar)->message <= WM_MOUSELAST)
+        {
+          if (is_app_blocked() && block_input)
+            {
+              ((MSG*)lpar)->message = WM_NULL;
+            }
+        }
+
+      if ( ((MSG*)lpar)->message >= WM_NCMOUSEMOVE &&
+           ((MSG*)lpar)->message <= WM_NCMOUSEMOVE + 0x10)
+        {
+          if (is_app_blocked() && block_input)
+            {
+              ((MSG*)lpar)->message = WM_NULL;
+            }
+        }
+    }
+
+  return CallNextHookEx(msg_hook, code, wpar, lpar);
+}
+
+
+
 /**********************************************************************
  * Initialisation
  **********************************************************************/
@@ -746,6 +796,7 @@ harpoon_init ( char imported_critical_file_list[][511], BOOL debug_harpoon )
               "Capture &Debug Messages" );
           AppendMenu( menu_popup, MF_SEPARATOR, 0,0 );
           AppendMenu( menu_popup, MF_STRING, IDM_MENU_CLEAR, "&Clear Display" );
+          AppendMenu( menu_popup, MF_STRING, IDM_MENU_UNHOOK, "&Unhook" );
           AppendMenu( menu, MF_POPUP, (UINT_PTR)menu_popup, "&Menu" );
         }
       else
@@ -873,6 +924,12 @@ harpoon_unhook ()
 {
   if_debug_send_message( "harpoon_unhook() called" );
 
+  if (msg_hook)
+    {
+      UnhookWindowsHookEx(msg_hook);
+      if_debug_send_message( "UnhookWindowsHookEx(msg_hook)" );
+      msg_hook = NULL;
+    }
   if (mouse_hook)
     {
       UnhookWindowsHookEx(mouse_hook);
@@ -989,7 +1046,22 @@ harpoon_hook_block_only(void)
   if (user_callback == NULL)
     {
       harpoon_unhook();
-      
+
+#ifdef _WIN64
+      if (msg_hook == NULL)
+        {
+          msg_hook = SetWindowsHookEx(WH_GETMESSAGE, 
+                                      harpoon_msg_block_hook, 
+                                      dll_handle, 0);
+          if ( msg_hook )
+            if_debug_send_message( "SetWindowsHookEx: WH_GETMESSAGE (success)" );
+          else
+            {
+              if_debug_send_message( "SetWindowsHookEx: WH_GETMESSAGE (failure)" );
+              harpoon_exit();
+            }
+        }
+#else
       if (mouse_hook == NULL)
         {
           mouse_hook = SetWindowsHookEx(WH_MOUSE,
@@ -1019,6 +1091,7 @@ harpoon_hook_block_only(void)
               harpoon_exit();
             }
         }
+#endif
     }
 }
 
@@ -1193,6 +1266,7 @@ buffer len is 1024 - 540 = ~480 max str len
       0, 0, 0, 0, 0
     };
   */
+
   GetLocalTime( &local );
 
   if( local.wSecond == wSecond_previous_call &&
@@ -1279,6 +1353,11 @@ static void debug_process_menu_selection( WORD idm )
           SetWindowText( debug_hwnd, "" );
           if_debug_send_message( "IDM_MENU_CLEAR" );
           break;
+
+      case IDM_MENU_UNHOOK:
+          harpoon_unhook();
+          if_debug_send_message( "IDM_MENU_UNHOOK" );
+          break;
     }
   return;
 }
