@@ -1,7 +1,7 @@
 /*
  * credits.h --- Workrave credits
  *
- * Copyright (C) 2007, 2008, 2009 Rob Caelers <robc@krandor.nl>
+ * Copyright (C) 2007, 2008, 2009, 2010 Rob Caelers <robc@krandor.nl>
  * All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
@@ -24,15 +24,15 @@
 #include <stdlib.h>
 
 const char *workrave_copyright =
-  "Copyright 2001-2008 Rob Caelers & Raymond Penners";
+  "Copyright 2001-2010 Rob Caelers & Raymond Penners";
 
 const char *workrave_authors[] = {
-  "Core developers:",
+  "Developers:",
   "Rob Caelers <robc@krandor.org>",
   "Raymond Penners <raymond@dotsphinx.com>",
-  "",
-  "Contributors:",
   "Ray Satiro <raysatiro@yahoo.com>",
+  "",
+  "Sounds:",
   "Ruairi Fullam",
   NULL
 };
@@ -52,8 +52,10 @@ const char *workrave_translators =
  /* he */           "Haggai Eran <he3@bezeqint.net>\n"
  /* hu-1 */         "ORY Mate <orymate@gmail.com>\n"
  /* hu-2 */         "Gabor Kelemen <kelemeng@gnome.hu>\n"
+ /* ga */           "Seanán Ó Coistín <seananoc@gmail.com>\n"
  /* it */           "Eraldo Girardi <eraldster@katamail.com>\n"
  /* ja */           "Masanobu Yokota <masanobu.yokota@nifty.com>\n"
+ /* ko */           "김재용 <nawane@naver.com>\n"
  /* lt */           "Artūras Šlajus <x11@h2o.sky.lt>\n"
  /* nl */           "Raymond Penners <raymond@dotsphinx.com>\n"
  /* no */           "Morten Lunde <morten.lunde@broadpark.no>\n"
