@@ -68,6 +68,7 @@ private:
   DBusMessage *NetworkLog(void *object, DBusMessage *message);
   DBusMessage *NetworkReconnect(void *object, DBusMessage *message);
   DBusMessage *NetworkDisconnect(void *object, DBusMessage *message);
+  DBusMessage *ReadingMode(void *object, DBusMessage *message);
   DBusMessage *Statistics(void *object, DBusMessage *message);
   DBusMessage *Exercises(void *object, DBusMessage *message);
   DBusMessage *RestBreak(void *object, DBusMessage *message);
@@ -112,7 +113,7 @@ org_workrave_ControlInterface_Stub::call(int method_num, void *object, DBusMessa
 {
   DBusMessage *ret = NULL;
   
-  if (method_num >=0 && method_num < 11 )
+  if (method_num >=0 && method_num < 12 )
     {
       DBusMethod m = method_table[method_num];
       if (m != NULL)
@@ -400,6 +401,56 @@ org_workrave_ControlInterface_Stub::NetworkDisconnect(void *object, DBusMessage
 
   
 
+DBusMessage *
+org_workrave_ControlInterface_Stub::ReadingMode(void *object, DBusMessage *message)
+{
+  DBusMessage *reply = NULL;
+
+  DBusMessageIter reader;
+  DBusMessageIter writer;
+  dbus_bool_t ok;
+
+  try
+    {
+      Menus *dbus_object = (Menus *) object;
+      
+      bool show       ;
+      
+      ok = dbus_message_iter_init(message, &reader);
+      if (!ok)
+        {
+          throw DBusSystemException("No parameters");
+        }
+        
+      get_bool(&reader, &show);
+
+      dbus_object->on_menu_reading(        show      );
+                                                              
+      reply = dbus_message_new_method_return(message);
+      if (reply == NULL)
+        {
+          throw DBusSystemException("Internal error");
+        }
+                                                              
+      dbus_message_iter_init_append(reply, &writer);
+      
+  }
+  catch (DBusException)
+    {
+      if (reply != NULL)
+        {
+          dbus_message_unref(reply);
+        }
+
+      throw;
+    }
+                                                          
+  return reply;
+
+}
+
+  
+
 DBusMessage *
 org_workrave_ControlInterface_Stub::Statistics(void *object, DBusMessage *message)
 {
@@ -636,6 +687,7 @@ org_workrave_ControlInterface_Stub::DBusMethod org_workrave_ControlInterface_Stu
   &org_workrave_ControlInterface_Stub::NetworkLog,
   &org_workrave_ControlInterface_Stub::NetworkReconnect,
   &org_workrave_ControlInterface_Stub::NetworkDisconnect,
+  &org_workrave_ControlInterface_Stub::ReadingMode,
   &org_workrave_ControlInterface_Stub::Statistics,
   &org_workrave_ControlInterface_Stub::Exercises,
   &org_workrave_ControlInterface_Stub::RestBreak,
@@ -662,6 +714,9 @@ DBusIntrospect org_workrave_ControlInterface_Stub::method_introspect[] = {
   { "NetworkDisconnect",
     ""
   },
+  { "ReadingMode",
+    "in\0b\0show\0"
+  },
   { "Statistics",
     ""
   },
