@@ -1,6 +1,6 @@
 // GUI.cc --- The WorkRave GUI
 //
-// Copyright (C) 2001 - 2009 Rob Caelers & Raymond Penners
+// Copyright (C) 2001 - 2010 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -30,6 +30,10 @@
 #include <glibmm/optiongroup.h>
 #include <glibmm/refptr.h>
 
+#ifdef PLATFORM_OS_WIN32_NATIVE
+#undef HAVE_UNISTD_H
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -65,11 +69,13 @@
 #include "Util.hh"
 #include "WindowHints.hh"
 #include "Locale.hh"
+#include "Session.hh"
 
 #if defined(PLATFORM_OS_WIN32)
 #include "W32AppletWindow.hh"
 #include <gdk/gdkwin32.h>
 #include <pbt.h>
+#include <wtsapi32.h>
 #endif
 
 #if defined(PLATFORM_OS_OSX)
@@ -92,7 +98,7 @@
 #endif
 
 #if defined(HAVE_DBUS)
-#if defined(PLATFORM_OS_WIN32_NATIVE)
+#if defined(interface)
 #undef interface
 #endif
 #include "DBus.hh"
@@ -131,7 +137,8 @@ GUI::GUI(int argc, char **argv) :
 #endif
   grab_handle(NULL),
   status_icon(NULL),
-  applet_control(NULL)
+  applet_control(NULL),
+  muted(false)
 {
   TRACE_ENTER("GUI:GUI");
 
@@ -175,7 +182,7 @@ GUI::~GUI()
 void
 GUI::restbreak_now()
 {
-  core->force_break(BREAK_ID_REST_BREAK, true);
+  core->force_break(BREAK_ID_REST_BREAK, BREAK_HINT_USER_INITIATED);
 }
 
 
@@ -189,10 +196,10 @@ GUI::main()
 
   if (!Glib::thread_supported())
     Glib::thread_init();
-#ifdef HAVE_DBUS
+
   Glib::OptionGroup *option_group = new Glib::OptionGroup(egg_sm_client_get_option_group());
   option_ctx.add_group(*option_group);
-#endif
+
   Gtk::Main *kit = NULL;
   try
     {
@@ -207,11 +214,11 @@ GUI::main()
   init_core();
   init_nls();
   init_platform();
-  init_session();
   init_debug();
   init_sound_player();
   init_multihead();
   init_dbus();
+  init_session();
   init_gui();
   
   on_timer();
@@ -314,6 +321,18 @@ GUI::on_timer()
 
   collect_garbage();
 
+  if (active_break_count == 0 && muted)
+    {
+      ICore *core = CoreFactory::get_core();
+      bool user_active = core->is_user_active();
+
+      if (user_active)
+        {
+          sound_player->set_mute(false);
+          muted = false;
+        }
+    }
+  
   return true;
 }
 
@@ -327,6 +346,7 @@ static void my_log_handler(const gchar *log_domain, GLogLevelFlags log_level,
 void
 GUI::init_platform()
 {
+  TRACE_ENTER("GUI::init_platform");
 #if defined(HAVE_KDE)
   init_kde();
 #endif
@@ -343,6 +363,7 @@ GUI::init_platform()
 #endif
   
   srand((unsigned int)time(NULL));
+  TRACE_EXIT();
 }
 
 
@@ -374,10 +395,9 @@ GUI::session_save_state_cb(EggSMClient *client, GKeyFile *key_file, GUI *gui)
 void
 GUI::init_session()
 {
+  TRACE_ENTER("GUI::init_session");
   EggSMClient *client = NULL;
-#ifdef HAVE_DBUS
   client = egg_sm_client_get();
-#endif
   if (client)
     {
       g_signal_connect(client,
@@ -389,6 +409,11 @@ GUI::init_session()
                        G_CALLBACK(session_save_state_cb),
                        this);
     }
+
+  session = new Session();
+  session->init();
+  
+  TRACE_EXIT();
 }
 
 
@@ -396,9 +421,8 @@ void
 GUI::cleanup_session()
 {
   EggSMClient *client = NULL;
-#ifdef HAVE_DBUS
+
   client = egg_sm_client_get();
-#endif
   if (client)
     {
       g_signal_handlers_disconnect_by_func(client,
@@ -430,6 +454,7 @@ void
 GUI::init_debug()
 {
 #if defined(NDEBUG)
+  TRACE_ENTER("GUI::init_debug");
   const char *domains[] = { NULL, "Gtk", "GLib", "Gdk", "gtkmm", "GLib-GObject" };
   for (unsigned int i = 0; i < sizeof(domains)/sizeof(char *); i++)
     {
@@ -437,7 +462,7 @@ GUI::init_debug()
                         (GLogLevelFlags) (G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),
                         my_log_handler, NULL);
     }
-
+  TRACE_EXIT();
 #endif
 }
 
@@ -495,7 +520,7 @@ GUI::init_nls()
 
   CoreFactory::get_configurator()->add_listener(GUIConfig::CFG_KEY_LOCALE, this);
 #endif
-  
+
   bindtextdomain(PACKAGE, locale_dir);
   bind_textdomain_codeset(PACKAGE, "UTF-8");
   textdomain(PACKAGE);
@@ -721,7 +746,7 @@ GUI::init_gtk_multihead()
                           irect.intersect(heads[k].geometry, overlap);
                         }
                     }
-
+                  
                   if (!overlap)
                     {
                       heads[count].screen = screen;
@@ -777,6 +802,8 @@ GUI::init_gui()
 
   // Status icon
   status_icon = new StatusIcon(*main_window);
+  status_icon->set_visible(GUIConfig::get_trayicon_enabled());
+  CoreFactory::get_configurator()->add_listener(GUIConfig::CFG_KEY_TRAYICON_ENABLED, this);
   
 #ifdef HAVE_DBUS
   DBus *dbus = CoreFactory::get_dbus();
@@ -809,8 +836,9 @@ GUI::init_dbus()
     {
       if (!dbus->is_owner())
         {
-          Gtk::MessageDialog dialog("Could not initialize Workrave. Is Workrave already running?",
+          Gtk::MessageDialog dialog(_("Workrave failed to start"),
                                     false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+          dialog.set_secondary_text(_("Is Workrave already running?"));
           dialog.show();
           dialog.run();
           exit(1);
@@ -819,7 +847,6 @@ GUI::init_dbus()
       try
         {
           dbus->register_service("org.workrave.Workrave.Activator");
-          
           dbus->register_object_path("/org/workrave/Workrave/UI");
 
           extern void init_DBusGUI(DBus *dbus);
@@ -863,6 +890,9 @@ GUI::init_sound_player()
   TRACE_ENTER("GUI:init_sound_player");
   try
     {
+      // Tell pulseaudio were are playing sound events
+      g_setenv("PULSE_PROP_media.role", "event", TRUE);
+      
       sound_player = new SoundPlayer(); /* LEAK */
       sound_player->init();
     }
@@ -875,19 +905,39 @@ GUI::init_sound_player()
 
 
 void
-GUI::core_event_notify(CoreEvent event)
+GUI::core_event_notify(const CoreEvent event)
 {
-  TRACE_ENTER_MSG("GUI::core_event_notify", event);
-  // FIXME: HACK
-  SoundPlayer::SoundEvent snd = (SoundPlayer::SoundEvent) event;
+  TRACE_ENTER_MSG("GUI::core_event_sound_notify", event);
+
   if (sound_player != NULL)
     {
-      TRACE_MSG("play");
-      sound_player->play_sound(snd);
+      if (event >= CORE_EVENT_SOUND_FIRST &&
+          event <= CORE_EVENT_SOUND_LAST)
+        {
+          SoundEvent snd = (SoundEvent) ( (int)event - CORE_EVENT_SOUND_FIRST);
+          TRACE_MSG("play");
+          sound_player->play_sound(snd);
+        }
+    }
+
+  if (event == CORE_EVENT_MONITOR_FAILURE)
+    {
+      string msg = _("Workrave could not monitor your keyboard and mouse activity.\n");
+
+#ifdef PLATFORM_OS_UNIX
+      msg += _("Make sure that the RECORD extention is enabled in the X server.");
+#endif
+      Gtk::MessageDialog dialog(_("Workrave failed to start"),
+                                false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+      dialog.set_secondary_text(msg);
+      dialog.show();
+      dialog.run();
+      terminate();
     }
   TRACE_EXIT();
 }
 
+
 void
 GUI::core_event_operation_mode_changed(const OperationMode m)
 {
@@ -895,8 +945,6 @@ GUI::core_event_operation_mode_changed(const OperationMode m)
     {
       status_icon->set_operation_mode(m);
     }
-
-  
   
   menus->resync();
 }
@@ -904,7 +952,7 @@ GUI::core_event_operation_mode_changed(const OperationMode m)
 void
 GUI::config_changed_notify(const std::string &key)
 {
-  (void) key;
+  TRACE_ENTER_MSG("GUI::config_changed_notify", key);
   
 #if defined(HAVE_LANGUAGE_SELECTION)
   if (key == GUIConfig::CFG_KEY_LOCALE)
@@ -915,6 +963,28 @@ GUI::config_changed_notify(const std::string &key)
       menus->locale_changed();
     }
 #endif
+  if (key == GUIConfig::CFG_KEY_TRAYICON_ENABLED)
+    {
+      if (status_icon != NULL)
+        {
+          bool tray = GUIConfig::get_trayicon_enabled();
+          status_icon->set_visible(tray);
+
+          if (!tray)
+            {
+              AppletControl *applet_control = get_applet_control();
+              if (applet_control != NULL)
+                {
+                  if (!applet_control->is_visible())
+                    {
+                      open_main_window();
+                    }
+                }
+            }
+        }
+    }
+    
+  TRACE_EXIT();
 }
 
 
@@ -945,7 +1015,7 @@ GUI::create_prelude_window(BreakId break_id)
 
 
 void
-GUI::create_break_window(BreakId break_id, bool user_initiated)
+GUI::create_break_window(BreakId break_id, BreakHint break_hint)
 {
   TRACE_ENTER_MSG("GUI::start_break_window", num_heads);
   hide_break_window();
@@ -955,9 +1025,10 @@ GUI::create_break_window(BreakId break_id, bool user_initiated)
   BreakWindow::BreakFlags break_flags = BreakWindow::BREAK_FLAGS_NONE;
   bool ignorable = GUIConfig::get_ignorable(break_id);
 
-  if (user_initiated && !ignorable)
+  if ( (break_hint & BREAK_HINT_USER_INITIATED) && !ignorable)
     {
-      break_flags = BreakWindow::BREAK_FLAGS_POSTPONABLE;
+      break_flags = ( BreakWindow::BREAK_FLAGS_POSTPONABLE |
+                      BreakWindow::BREAK_FLAGS_USER_INITIATED);
     }
   else if (ignorable)
     {
@@ -965,6 +1036,11 @@ GUI::create_break_window(BreakId break_id, bool user_initiated)
                        BreakWindow::BREAK_FLAGS_SKIPPABLE);
     }
 
+  if (break_hint & BREAK_HINT_NATURAL_BREAK)
+    {
+      break_flags |=  (BreakWindow::BREAK_FLAGS_NO_EXERCISES | BreakWindow::BREAK_FLAGS_NATURAL);
+    }
+  
   active_break_id = break_id;
 
   for (int i = 0; i < num_heads; i++)
@@ -979,6 +1055,17 @@ GUI::create_break_window(BreakId break_id, bool user_initiated)
 
   active_break_count = num_heads;
 
+  if (break_id == BREAK_ID_REST_BREAK ||
+      break_id == BREAK_ID_DAILY_LIMIT)
+    {
+      bool mute = false;
+      CoreFactory::get_configurator()->get_value(SoundPlayer::CFG_KEY_SOUND_MUTE, mute);
+      if (mute)
+        {
+          muted = !sound_player->set_mute(true);
+        }
+    }
+  
   TRACE_EXIT();
 }
 
@@ -1324,10 +1411,15 @@ GUI::map_from_head(int &x, int &y, int head)
 
 
 bool
-GUI::bound_head(int &x, int &y, int width, int height, int head)
+GUI::bound_head(int &x, int &y, int width, int height, int &head)
 {
   bool ret = false;
 
+  if (head >= num_heads)
+    {
+      head = 0;
+    }
+  
   HeadInfo &h = get_head(head);
   if (x < - h.get_width())
     {
@@ -1452,6 +1544,7 @@ GUI::is_status_icon_visible() const
   return ret;
 }
 
+
 #if defined(PLATFORM_OS_WIN32)
 void
 GUI::win32_init_filter()
@@ -1459,8 +1552,13 @@ GUI::win32_init_filter()
   GtkWidget *window = (GtkWidget *)main_window->gobj();
   GdkWindow *gdk_window = window->window;
   gdk_window_add_filter(gdk_window, win32_filter_func, this);
+
+  HWND hwnd = (HWND) GDK_WINDOW_HWND(gdk_window);
+  
+  WTSRegisterSessionNotification(hwnd, NOTIFY_FOR_THIS_SESSION);
 }
 
+
 GdkFilterReturn
 GUI::win32_filter_func (void     *xevent,
                         GdkEvent *event,
@@ -1474,6 +1572,19 @@ GUI::win32_filter_func (void     *xevent,
   GdkFilterReturn ret = GDK_FILTER_CONTINUE;
   switch (msg->message)
     {
+    case WM_WTSSESSION_CHANGE:
+      {
+        if (msg->wParam == WTS_SESSION_LOCK)
+          {
+            gui->session->set_idle(true);
+          }
+        if (msg->wParam == WTS_SESSION_UNLOCK)
+          {
+            gui->session->set_idle(false);
+          }
+      }
+      break;
+      
     case WM_POWERBROADCAST:
       {
         TRACE_MSG("WM_POWERBROADCAST " << msg->wParam << " " << msg->lParam);
@@ -1538,10 +1649,12 @@ GUI::win32_filter_func (void     *xevent,
         }
     }
 
+#ifndef USE_W32STATUSICON
   if (ret != GDK_FILTER_REMOVE && gui->status_icon)
     {
       ret = gui->status_icon->win32_filter_func(xevent, event);
     }
+#endif
 
   TRACE_EXIT();
   return ret;
