@@ -1,6 +1,6 @@
 // MainWindow.cc --- Main info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -97,6 +97,9 @@ MainWindow::MainWindow() :
 {
 #ifdef PLATFORM_OS_UNIX
   leader = NULL;
+#endif
+#ifdef PLATFORM_OS_WIN32
+  show_retry_count = 0;
 #endif
   init();
 }
@@ -299,7 +302,14 @@ MainWindow::update()
 
 void MainWindow::on_activate()
 {
-  open_window();
+  if (GTK_WIDGET_VISIBLE(GTK_WIDGET(gobj())))
+    {
+      close_window();
+    }
+  else
+    {
+      open_window();
+    }
 }
 
 
@@ -346,12 +356,14 @@ MainWindow::open_window()
 void
 MainWindow::close_window()
 {
+  TRACE_ENTER("MainWindow::close_window");
 #ifdef PLATFORM_OS_WIN32
   win32_show(false);
 #elif defined(PLATFORM_OS_OSX)
   hide_all();
 #else
   GUI *gui = GUI::get_instance();
+
   if (applet_active || gui->is_status_icon_visible())
     {
       hide_all();
@@ -361,6 +373,7 @@ MainWindow::close_window()
       iconify();
     }
 #endif
+  TRACE_EXIT();
 }
 
 
@@ -488,28 +501,63 @@ MainWindow::set_start_in_tray(bool b)
 void
 MainWindow::win32_show(bool b)
 {
+  TRACE_ENTER("MainWindow::win32_show");
+  bool retry = false;
+
   // Gtk's hide() seems to quit the program.
   GtkWidget *window = Gtk::Widget::gobj();
   GdkWindow *gdk_window = window->window;
   HWND hwnd = (HWND) GDK_WINDOW_HWND(gdk_window);
   ShowWindow(hwnd, b ? SW_SHOWNORMAL : SW_HIDE);
 
-	if( b )
-	// show main window
-	{
-		/**/
-		present(); //works in all my testing. does this not always work?
-		/**/
-		if( hwnd != GetForegroundWindow() )
-		{
-			SetWindowPos( hwnd, HWND_TOP, 0, 0, 0, 0, 
-				SWP_NOMOVE | SWP_NOSIZE | SWP_FRAMECHANGED | SWP_SHOWWINDOW );
-			//BringWindowToTop( hwnd );
-			SetForegroundWindow( hwnd );
-		}
-	}
+	if (b)
+	  {
+		  present();
+
+		  if (hwnd != GetForegroundWindow())
+		    {
+          if (show_retry_count == 0)
+            {
+              show_retry_count = 20;
+            }
+          else
+            {
+              show_retry_count--;
+            }
+
+          TRACE_MSG("2 " << show_retry_count);
+          retry = true;  
+        }
+    }
+
+  if (retry)
+    {
+      if (show_retry_count > 0)
+        {
+          Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::win32_show_retry), 50);
+        }
+    }
+  else
+    {
+      show_retry_count = 0;
+    }
+  TRACE_EXIT();
 }
 
+bool
+MainWindow::win32_show_retry()
+{
+  TRACE_ENTER("MainWindow::win32_show_retry");
+  if (show_retry_count > 0)
+  {
+     TRACE_MSG("retry");
+     win32_show(true);
+  }
+  TRACE_EXIT();
+  return false;
+}
+
+
 
 void
 MainWindow::win32_init()
@@ -695,7 +743,6 @@ MainWindow::locate_window(GdkEventConfigure *event)
   (void) event;
 
   Glib::RefPtr<Gdk::Window> window = get_window();
-  TRACE_MSG(window->get_state());
   if ((window->get_state() & (Gdk::WINDOW_STATE_ICONIFIED | Gdk::WINDOW_STATE_WITHDRAWN)) != 0)
     {
       TRACE_EXIT();
@@ -710,8 +757,6 @@ MainWindow::locate_window(GdkEventConfigure *event)
       y = event->y;
       width = event->width;
       height = event->height;
-
-      TRACE_MSG("main window2 = " << x << " " << y);
     }
   else
 #endif
@@ -724,8 +769,6 @@ MainWindow::locate_window(GdkEventConfigure *event)
       on_size_request(&req);
       width = req.width;
       height = req.height;
-
-      TRACE_MSG("main window1 = " << x << " " << y);
     }
 
 
@@ -747,10 +790,10 @@ MainWindow::locate_window(GdkEventConfigure *event)
       window_relocated_location.set_y(y);
 
       int head = GUI::get_instance()->map_to_head(x, y);
-      TRACE_MSG("main window head = " << x << " " << y);
-      // Stores location relative to origin of current head.
+      TRACE_MSG("main window head = " << x << " " << y << " " << head);
 
       bool rc = GUI::get_instance()->bound_head(x, y, width, height, head);
+      TRACE_MSG("main window bounded = " << x << " " << y);
 
       window_head_location.set_x(x);
       window_head_location.set_y(y);
@@ -831,7 +874,7 @@ LRESULT CALLBACK
 MainWindow::win32_window_proc(HWND hwnd, UINT uMsg, WPARAM wParam,
                               LPARAM lParam)
 {
-  TRACE_ENTER("MainWindow::win32_window_proc");
+  TRACE_ENTER_MSG("MainWindow::win32_window_proc", uMsg << " " << wParam);
   TRACE_EXIT();
   return DefWindowProc(hwnd, uMsg, wParam, lParam);
 }
