@@ -1,6 +1,6 @@
 // WorkraveApplet.cc
 //
-// Copyright (C) 2002, 2003, 2005, 2006, 2007, 2008, 2009 Rob Caelers & Raymond Penners
+// Copyright (C) 2002, 2003, 2005, 2006, 2007, 2008, 2009, 2010 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -73,6 +73,7 @@ workrave_applet_init(WorkraveApplet *applet)
   applet->socket_id = 0;
   applet->orientation = 0;
   applet->last_showlog_state = FALSE;
+  applet->last_reading_mode_state = FALSE;
   applet->last_mode = 0; 
   applet->applet = NULL;
   
@@ -348,7 +349,11 @@ verb_about(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 
   gtk_show_about_dialog(NULL,
                         "name", "Workrave",
+#ifdef GIT_VERSION                        
+                        "version", PACKAGE_VERSION "\n(" GIT_VERSION ")",
+#else                        
                         "version", PACKAGE_VERSION,
+#endif                        
                         "copyright", workrave_copyright,
                         "website", "http://www.workrave.org",
                         "website_label", "www.workrave.org",
@@ -385,7 +390,6 @@ verb_preferences(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     {
       dbus_g_proxy_begin_call(g_applet->ui, "Preferences", dbus_callback, NULL, NULL,
                              G_TYPE_INVALID);
-
     }
 }
 
@@ -402,8 +406,6 @@ verb_exercises(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     {
       dbus_g_proxy_begin_call(g_applet->ui, "Exercises", dbus_callback, NULL, NULL,
                               G_TYPE_INVALID);
-
-
     }
 }
 
@@ -419,8 +421,6 @@ verb_statistics(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     { 
       dbus_g_proxy_begin_call(g_applet->ui, "Statistics", dbus_callback, NULL, NULL,
                              G_TYPE_INVALID);
-
-
     }
 }
 
@@ -436,8 +436,6 @@ verb_restbreak(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     {
       dbus_g_proxy_begin_call(g_applet->ui, "RestBreak", dbus_callback, NULL, NULL,
                              G_TYPE_INVALID);
-
-
     }
 }
 
@@ -456,8 +454,6 @@ verb_connect(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
       dbus_g_proxy_begin_call(g_applet->ui, "NetworkConnect", dbus_callback, NULL, NULL,
                              G_TYPE_INVALID);
 
-
-
     }
 }
 
@@ -474,8 +470,6 @@ verb_disconnect(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     {
       dbus_g_proxy_begin_call(g_applet->ui, "NetworkDisconnect", dbus_callback, NULL, NULL,
                              G_TYPE_INVALID);
-
-
     }
 }
 
@@ -491,8 +485,6 @@ verb_reconnect(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     {
       dbus_g_proxy_begin_call(g_applet->ui, "NetworkReconnect", dbus_callback, NULL, NULL,
                              G_TYPE_INVALID);
-
-
     }
 }
 
@@ -510,8 +502,6 @@ verb_quit(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     {
       dbus_g_proxy_begin_call(g_applet->ui, "Quit", dbus_callback, NULL, NULL,
                              G_TYPE_INVALID);
-
-
     }
 }
 
@@ -571,23 +561,42 @@ change_background(PanelApplet * widget,
   long xid = 0;
   GValueArray *val = g_value_array_new(4);
 
+  if (type == PANEL_NO_BACKGROUND)
+    {
+      GtkStyle *style = gtk_widget_get_style(GTK_WIDGET(widget));
+
+      if (style->bg_pixmap[GTK_STATE_NORMAL])
+        {
+          pixmap = style->bg_pixmap[GTK_STATE_NORMAL];
+          type = PANEL_PIXMAP_BACKGROUND;
+        }
+      else
+        {
+          color = &style->bg[GTK_STATE_NORMAL];
+          if (color != NULL)
+            {
+              type = PANEL_COLOR_BACKGROUND;
+            }
+        }
+    }
+  
   if (type == PANEL_COLOR_BACKGROUND && color != NULL)
     {
-      g_value_array_prepend(val, NULL);
+      g_value_array_append(val, NULL);
       g_value_init(g_value_array_get_nth(val, 0), G_TYPE_UINT);
       g_value_set_uint(g_value_array_get_nth(val, 0), color->pixel);
  
-      g_value_array_prepend(val, NULL);
-      g_value_init(g_value_array_get_nth(val, 0), G_TYPE_UINT);
-      g_value_set_uint(g_value_array_get_nth(val, 0), color->red);
+      g_value_array_append(val, NULL);
+      g_value_init(g_value_array_get_nth(val, 1), G_TYPE_UINT);
+      g_value_set_uint(g_value_array_get_nth(val, 1), color->red);
       
-      g_value_array_prepend(val, NULL);
-      g_value_init(g_value_array_get_nth(val, 0), G_TYPE_UINT);
-      g_value_set_uint(g_value_array_get_nth(val, 0), color->green);
+      g_value_array_append(val, NULL);
+      g_value_init(g_value_array_get_nth(val, 2), G_TYPE_UINT);
+      g_value_set_uint(g_value_array_get_nth(val, 2), color->green);
 
-      g_value_array_prepend(val, NULL);
-      g_value_init(g_value_array_get_nth(val, 0), G_TYPE_UINT);
-      g_value_set_uint(g_value_array_get_nth(val, 0), color->blue);
+      g_value_array_append(val, NULL);
+      g_value_init(g_value_array_get_nth(val, 3), G_TYPE_UINT);
+      g_value_set_uint(g_value_array_get_nth(val, 3), color->blue);
     }
   else
     {
@@ -618,10 +627,10 @@ change_background(PanelApplet * widget,
   if (g_applet->support != NULL && workrave_is_running())
     {
       dbus_g_proxy_begin_call(g_applet->support, "SetBackground", dbus_callback, NULL, NULL,
-                             G_TYPE_UINT, type,
-                             G_TYPE_VALUE_ARRAY, val,
-                             G_TYPE_UINT, xid,
-                             G_TYPE_INVALID);
+                              G_TYPE_UINT, type,
+                              G_TYPE_VALUE_ARRAY, val,
+                              G_TYPE_UINT, xid,
+                              G_TYPE_INVALID);
       
 
     }
@@ -659,7 +668,7 @@ button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer data)
     {
       if (g_applet->support != NULL && workrave_is_running())
         {
-          dbus_g_proxy_begin_call(g_applet->support, "ButtonPressed", dbus_callback, NULL, NULL,
+          dbus_g_proxy_begin_call(g_applet->support, "ButtonClicked", dbus_callback, NULL, NULL,
                                   G_TYPE_UINT, event->button, G_TYPE_INVALID);
 
           ret = TRUE;
@@ -695,6 +704,31 @@ showlog_callback(BonoboUIComponent *ui, const char *path, Bonobo_UIComponent_Eve
     }
 }
 
+static void
+reading_mode_callback(BonoboUIComponent *ui, const char *path, Bonobo_UIComponent_EventType type,
+                      const char *state, gpointer user_data)
+{
+  gboolean new_state;
+
+  if (state == NULL || strcmp(state, "") == 0)
+    {
+      /* State goes blank when component is removed; ignore this. */
+      return;
+    }
+
+  new_state = strcmp(state, "0") != 0;
+
+  if (1)
+    {
+      g_applet->last_reading_mode_state = new_state;
+
+      if (g_applet->ui != NULL && workrave_is_running())
+        {
+          dbus_g_proxy_begin_call(g_applet->ui, "ReadingMode", dbus_callback, NULL, NULL,
+                                 G_TYPE_BOOLEAN, new_state, G_TYPE_INVALID);
+        }
+    }
+}
 
 static void
 mode_callback(BonoboUIComponent *ui, const char *path, Bonobo_UIComponent_EventType type,
@@ -886,6 +920,7 @@ workrave_applet_fill(PanelApplet *applet)
   bonobo_ui_component_add_listener(ui, "Normal", mode_callback, NULL);
   bonobo_ui_component_add_listener(ui, "Suspended", mode_callback, NULL);
   bonobo_ui_component_add_listener(ui, "Quiet", mode_callback, NULL);
+  bonobo_ui_component_add_listener(ui, "ReadingMode", reading_mode_callback, NULL);
 
   panel_applet_set_flags(PANEL_APPLET(applet),
                          PANEL_APPLET_EXPAND_MINOR);
